--
-- mpm_ax.vhd
--
-- This file implements a variable width "Quad-port" memory or a 
-- "pseudo-true dual port" memory based on the embedded memory blocks
-- avaialble in Actel's AX Family of FPGAs. This wrapper supports depth
-- of 128, 256, 512, 1024, 2048, or 4096 words based on the variable
-- aspect ratio of the AX memory blocks.  The user must gang blocks
-- together or modify this wrapper to accomodate deeper memories.
--
-- The memory implemented in this file offers 4 access ports: 2 write
-- ports and 2 read ports (QPM mode) or 2 write/read ports (DPM mode).
--
-- Two clocks - one full rate clock (WR_2xCLK) and one half-rate clock
-- (WR_CLK) are shared by the write ports.
--
-- Likewise, two clocks - one full rate clock (RD_2xCLK) and one
-- half-rate clock (RD_CLK) are shared by the read ports.
--
-- Write ports:
--  Data, address, and enables for both write ports are optionally 
--  registered (WMODE = 1) with the rising edge of WR_CLK. Then during
--  the high portion of WR_CLK, the values for WPa are muxed to the
--  memory blocks and settle, then data is written into the memory on
--  the rising edge of WR_2xCLK (ie the falling edge of WR_CLK).  During
--  the low portion of WR_CLK, the values for WPb are muxed to the memory
--  and allowed to settle. Then on the rising-edge of WR_2xCLK (ie: the
--  next rising-edge of WR_CLK), these values are written into the memory
--  while the next write-port values are captured.
--
-- READ ports (DPM mode:PMODE = 0)
--
-- When PMODE is set to '0' (DPM), the read address inputs and read clock are
-- unused and the write addresses are used to read the memory.  IF OREG = '0',
-- the data outputs propagate directly to the rd_out ports; otherwise, when
-- OREG = '1', RDa is registered on the falling-edge of WR_CLK, and RDb is
-- registered on the rising-edge of WR_CLK.  The read enable inputs are used
-- as inputs to enable the output regsters.
--
-- NOTE: the RAM64k36 is used as the basic memory for this wrapper, so
-- in DPM mode, you should not allow simultaneous write and read operations;
-- unless the clock period is sufficiently long to ensure that the data
-- being written into the memory will settle and propagate to the outputs
-- within 1/2 of the clock cycle.
--
-- This same restriction applies in QPM mode, if WMODE and RMODE are the
-- same, and the port_x write and read addresses are equal. 
--
-- Care should be taken in QPM mode to understand the impact of setting
-- WMODE/RMODE/OREG and possible write/read conflicts that may result.
--
-- When PMODE = 1 (QPM), if RMODE is set to "1", the read address and enables
-- for both read ports are registered with the rising edge of the read
-- clock (RD_CLK).  During the high portion of RD_CLK, the address for RDa
-- is muxed to the memory block.  On the next-rising edge of rd_2xclk (the
-- falling-edge of RD_CLK) this address/enables are latched into the memory.
-- RDa_out propagates through the memory block, and if OREG is set to "0"
-- the result is wired directly to RDa_OUT (transparent outputs), otherwise,
-- if OREG = 1, an register will capture the result for RDa on the rising-
-- edge of RD_CLK.  Then during the low portion of RD_CLK, the address for
-- RDb is muxed to the memory blocks and is registered on the next rising-edge
-- of rd_2xclk (rising-edge of rd_clk) and propagates from the memory.
-- Again, if OMODE = 0, then the result is wired directly to RDb_OUT, 
-- otherwise register captures the result on the next falling-edge of rd_clk.
--
--  If RMODE is set to 0, the address and enables are not registered
--  and the mux described above is provided for Raddr_a and Raddr_b.
--
-- WIDTH/DEPTH
--
-- Because the embedded memories in the AX Family have variable aspect
-- ratio - this wrapper supports independent width and depth for write
-- and read ports.  The user specifies the depth and width for the write
-- ports, and depth only for the read ports - read port width is 
-- derived from the other values.
--
-- This wrapper supports write and read depth of 128, 256, 512, 1024,
-- 2048, or 4096 address locations (ie: one block in depth); to support
-- deeper memories, the user must modify this wrapper to use the 
-- DEPTH inputs on the embedded memory instances.
--
-- This wrapper supports write data width in integer multiples of the
-- word-width associated with the specified write depth.  E.G. if the
-- specified write depth is 256, the width may be any multiple of 
-- 18-bits upto a maximum of 288-bits.
--
-- NOTE that the wrapper can not 'valibate' the GENERIC values the
-- the user specifies, so care must be taken to specify correct
-- width and depth parameters for the desired result.
--
-- PARAMTER    VALUE    DESCRIPTION
--
--  PMODE        0      Dual-port operation    (Default)
--               1      Quad-port operation  
--
--  WMODE        0      No register, only mux logic for Waddr (Default)
--               1      Register, then muliplex Waddr, Wenb, and Wdata
--
--  RMODE        0      No register, only mux logic for Raddr (Default)
--               1      Register, then muliplex Raddr
--
--  OREG         0      Transparent output mode  (default)
--               1      Registered  output mode
--
--  WR_WIDTH     XXX    Write port data width
--                       NOTE: must be a multiple of the memory width
--                       associated with the specified wr_depth:
--                       (36,18,9,4,2,1)
--  WR_DEPTH     XXX    Write port depth (128,256,512,1024,2048,4096)
--  RD_DEPTH     XXX    Read port depth  (128,256,512,1024,2048,4096)
--          
--  Here's a sample instantiation of the block (cut and paste into
--  your higher-level VHDL code):
--
--  QPM0: mpm_ax
--  GENERIC MAP (PMODE => 0, WMODE => 0, RMODE => 0, OREG => 0, 
--               WR_DEPTH => 256, WR_WIDTH => 36, RD_DEPTH => 512);
--
--  PORT MAP    (reset_n  => <your reset>,
--               wr_2xclk => <2x write clock>,
--               wr_clk   => <write port data/enable clock>,
--
--               a_wren    => <active high data enable for write>,
--               a_wadr   => <write/dpm address bus  (8-bits)>,
--               a_wdata  => <write data bus (variable width)>,
--   
--               b_wren    => <active high data enable for write>,
--               b_wadr   => <write/dpm address bus  (8-bits)>,
--               b_wdata  => <write data bus (variable width)>,
--
--               rd_2xclk => <2X read clock   (QPM mode)>,
--               rd_clk   => <read data clock (QPM mode)>,
--
--               a_rden    => <output register enable A>,
--               a_radr   => <output address bus 8-bits (QPM mode)>,
--               a_rdata  => <output data bus (variable width)>,
--
--               b_rden    => <output register enable B>,
--               b_radr   => <output address bus 8-bits (QPM mode)>,
--               b_rdata  => <output data bus (variable width)>);
--
------------------------------------------------------------------------
--
--                  Copyright 2002 Actel corporation
--
------------------------------------------------------------------------
--
-- Version 1.0  11/02/02 J.Vorgert - Initial release
-- Version 1.1  03/09/03 J.Vorgert - No changes to this file.
-- Version 1.2  04/12/03 J.Vorgert - corrected several issues with DPM
--                                   mode and bus mis-matches.
------------------------------------------------------------------------
--
-- This package declares types for the generic parameters so that
-- you can specify values that convey information on each instance.
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

-- This package contains functions used to translate the 
-- generics when calculating bits widths & structure sizes.

package GTYPES is
 function get_adr (DEPTH: INTEGER) return integer;
 function get_max (RDEPTH: INTEGER; WDEPTH: INTEGER) return integer;
 function get_bits(DEPTH: INTEGER) return integer;
 function get_val (DEPTH: INTEGER) return std_logic_vector;
end package GTYPES;

Package body GTYPES is

-- number of valid address bits based on depth (N-1)

 function get_adr(DEPTH: INTEGER) return integer is
  variable result: integer;
 begin
    case (depth) is
     when 128    => result :=  6;
     when 256    => result :=  7;
     when 512    => result :=  8;
     when 1024   => result :=  9;
     when 2048   => result := 10;
     when 4096   => result := 11;
     when others => result := 0;  -- will cause errors
    end case;
  return result;
 end function;

-- Returns the larger of the two values

 function get_max(PMODE: INTEGER; RDEPTH: INTEGER; WDEPTH: INTEGER) return integer is
  variable result: integer;
 begin
   IF (PMODE = 0 AND RDEPTH > WDEPTH) then
      result := get_adr(RDEPTH);
   ELSE
      result := get_adr(WDEPTH);
   END IF;
  return result;
 end function;

-- Width #bits based on selected depth

 function get_bits(DEPTH: INTEGER) return integer is
  variable result: integer;
 begin
    case (depth) is
     when 128    => result := 36;
     when 256    => result := 18;
     when 512    => result := 9;
     when 1024   => result := 4;
     when 2048   => result := 2;
     when 4096   => result := 1;
     when others => result := 0;  -- will cause errors
    end case;
  return result;
 end function;
 
-- static value to set on RAM for a given depth

 function get_val(DEPTH: INTEGER) return std_logic_vector is
  variable result: std_logic_vector(2 downto 0);
 begin
    case (depth) is
     when 128    => result := "101";
     when 256    => result := "100";
     when 512    => result := "011";
     when 1024   => result := "010";
     when 2048   => result := "001";
     when 4096   => result := "000";
     when others => result := "111";  -- will cause errors
    end case;
  return result;
 end function;

end package body GTYPES;

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

use work.GTYPES.all;

-- depending on the tools suite used, the AX library
-- may need to be referenced for VITAL simulation models.
--
-- NOTE: Integer types are used for all GENERIC declarations 
-- in order to include synopsys support.

entity mpm_ax is
    generic (PMODE      : INTEGER range 0 to 1 := 0;       -- DPM / QPM
             WMODE      : INTEGER range 0 to 1 := 0;       -- NO  / Pipe-line before mux
             RMODE      : INTEGER range 0 to 1 := 0;       -- NO  / Pipe-line before mux
             OREG       : INTEGER range 0 to 1 := 0;       -- NO  / Pipe-line outputs

             WR_WIDTH   : INTEGER := 36;                   -- 36 ,18 ,9  ,4   ,2   ,1
             WR_DEPTH   : INTEGER range 1 to 4096 := 128;  -- 128,256,512,1024,2048,4096
             RD_DEPTH   : INTEGER range 1 to 4096 := 128); -- 128,256,512,1024,2048,4096
 
    port( reset_n   : in  std_logic;  -- active low
          
          wr_2xclk  : in  std_logic;
          wr_clk    : in  std_logic;

          a_wren    : in  std_logic; 
          a_wadr    : in  std_logic_vector(get_max(PMODE, RD_DEPTH, WR_DEPTH) downto 0);
          a_wdata   : in  std_logic_vector(WR_WIDTH - 1 downto 0);

          b_wren    : in  std_logic; 
          b_wadr    : in  std_logic_vector(get_max(PMODE, RD_DEPTH, WR_DEPTH) downto 0);
          b_wdata   : in  std_logic_vector(WR_WIDTH - 1 downto 0);

          rd_2xclk  : in  std_logic;
          rd_clk    : in  std_logic;

          a_rden    : in  std_logic; 
          a_radr    : in  std_logic_vector(get_adr(RD_DEPTH) downto 0);
          a_rdata   : out std_logic_vector(((WR_WIDTH / get_bits(WR_DEPTH)) *
                                             get_bits(RD_DEPTH)) - 1 downto 0);

          b_rden    : in  std_logic; 
          b_radr    : in  std_logic_vector(get_adr(RD_DEPTH) downto 0);
          b_rdata   : out std_logic_vector(((WR_WIDTH / get_bits(WR_DEPTH)) *
                                             get_bits(RD_DEPTH)) - 1 downto 0));
end mpm_ax;

architecture RTL of mpm_ax is

-- AX embedded RAM

component RAM64K36 port(
 WRAD0, WRAD1,  WRAD2,  WRAD3,  WRAD4,  WRAD5,  WRAD6, WRAD7, WRAD8,
 WRAD9, WRAD10, WRAD11, WRAD12, WRAD13, WRAD14, WRAD15 :in std_logic;

 WD0,  WD1,  WD2,  WD3,  WD4,  WD5,  WD6,  WD7,  WD8,  WD9,  WD10,
 WD11, WD12, WD13, WD14, WD15, WD16, WD17, WD18, WD19, WD20, WD21,
 WD22, WD23, WD24, WD25, WD26, WD27, WD28, WD29, WD30, WD31, WD32,
 WD33, WD34, WD35 :in std_logic;

 WW0, WW1, WW2, WEN, WCLK       :in std_logic;
 DEPTH0, DEPTH1, DEPTH2, DEPTH3 :in std_logic;
 RW0, RW1, RW2, REN, RCLK       :in std_logic;

 RDAD0, RDAD1,  RDAD2,  RDAD3,  RDAD4,  RDAD5,  RDAD6, RDAD7, RDAD8,
 RDAD9, RDAD10, RDAD11, RDAD12, RDAD13, RDAD14, RDAD15 :in std_logic;

 RD0,  RD1,  RD2,  RD3,  RD4,  RD5,  RD6,  RD7,  RD8,  RD9,  RD10,
 RD11, RD12, RD13, RD14, RD15, RD16, RD17, RD18, RD19, RD20, RD21,
 RD22, RD23, RD24, RD25, RD26, RD27, RD28, RD29, RD30, RD31, RD32,
 RD33, RD34, RD35 :out std_logic);
end component;

SIGNAL wren_a   : std_logic;
SIGNAL wadr_a   : std_logic_vector(get_max(PMODE, RD_DEPTH, WR_DEPTH) downto 0);
SIGNAL wdata_a  : std_logic_vector(WR_WIDTH - 1 downto 0);
SIGNAL wren_b   : std_logic;
SIGNAL wadr_b   : std_logic_vector(get_max(PMODE, RD_DEPTH, WR_DEPTH) downto 0);
SIGNAL wdata_b  : std_logic_vector(WR_WIDTH - 1 downto 0);
SIGNAL wrenb    : std_logic;
SIGNAL wadr     : std_logic_vector( 11 downto 0);
SIGNAL wdata    : std_logic_vector((36*(WR_WIDTH/get_bits(WR_DEPTH)))-1 downto 0);

SIGNAL rden_a   : std_logic;
SIGNAL rden_b   : std_logic;
SIGNAL rden     : std_logic;
SIGNAL a_oen    : std_logic;
SIGNAL b_oen    : std_logic;

SIGNAL radr_a   : std_logic_vector(get_adr(RD_DEPTH) downto 0);
SIGNAL radr_b   : std_logic_vector(get_adr(RD_DEPTH) downto 0);
SIGNAL radr     : std_logic_vector( 11 downto 0);
SIGNAL rdata    : std_logic_vector((36*(WR_WIDTH/get_bits(WR_DEPTH)))-1 downto 0);
SIGNAL rd_data  : std_logic_vector((((WR_WIDTH / get_bits(WR_DEPTH)) * 
                                      get_bits(RD_DEPTH)) - 1) downto 0);

SIGNAL GND         : std_logic;
SIGNAL w_vec,r_vec : std_logic_vector(2 downto 0);
SIGNAL read_clk    : std_logic;

begin

GND <= '0'; -- used to tie off un-used inputs to memories

w_vec <= get_val(WR_DEPTH); -- get contol vectors for memories
r_vec <= get_val(RD_DEPTH);

--
-- WRITE PORTS
--

A: if (WMODE = 1) generate
B: process(reset_n, wr_clk)
    begin
     if (reset_n = '0') then
 
      wren_a   <= '0';
      wadr_a   <= (OTHERS => '0');
      wdata_a  <= (OTHERS => '0');

      wren_b   <= '0';
      wadr_b   <= (OTHERS => '0');
      wdata_b  <= (OTHERS => '0');

     elsif (wr_clk'event and wr_clk = '1') then

      wren_a   <= a_wren  after 1 ns;
      wadr_a   <= a_wadr  after 1 ns;
      wdata_a  <= a_wdata after 1 ns;

      wren_b   <= b_wren  after 1 ns;
      wadr_b   <= b_wadr  after 1 ns;
      wdata_b  <= b_wdata after 1 ns;

     end if;
   end process;
end generate;

-- 
-- Otherwise, just pass 'em through to the mux
--

C: if (WMODE /= 1) generate
      wren_a   <= a_wren;
      wadr_a   <= a_wadr;
      wdata_a  <= a_wdata;

      wren_b   <= b_wren;
      wadr_b   <= b_wadr;
      wdata_b  <= b_wdata;
end generate;

--
-- Multiplex the write address/enables to the memory
--

wrenb <= wren_a when (wr_clk = '1') else wren_b;

wadr(get_adr(WR_DEPTH) downto 0) <= wadr_a(get_adr(WR_DEPTH) downto 0)
     when (wr_clk = '1') else wadr_b(get_adr(WR_DEPTH) downto 0);

-- Tie off unused address bits to gnd

D: if (get_adr(WR_DEPTH) < 11) generate
 wadr(11 downto get_adr(WR_DEPTH)+1) <= (OTHERS => '0');
end generate;

-- The write data needs to be "distributed" across
-- the total of 288 possible bits based on WR_DEPTH
-- and allocation of 36 bits of the write bus to 
-- each physical memory.  For example, with WR_DEPTH
-- equals 512, only 9 bits of each 36 are connected,
-- for WR_WIDTH of 36, four physical memories are
-- used, so bits 0-8, 36-43, 72-79, and 108-115 are
-- mapped to the 36 wdata_x bits.
--

E: for i in 0 to ((WR_WIDTH/get_bits(WR_DEPTH))-1) generate
   Y: for j in 0 to (get_bits(WR_DEPTH)-1) generate       
        wdata((36*i)+j) <= wdata_a((get_bits(WR_DEPTH)*i)+j) when
	                      (wr_clk = '1') else
                           wdata_b((get_bits(WR_DEPTH)*i)+j);
   end generate;
end generate;

-- Tie off unused data inputs to gnd

F: if (get_bits(WR_DEPTH) < 36) generate
   G: for i in 0 to ((WR_WIDTH/get_bits(WR_DEPTH))-1) generate
      H: for j in (get_bits(WR_DEPTH)) to 35 generate       
         wdata((36*i)+j) <= GND;
       end generate;
    end generate;
end generate;

--
-- READ PORTS
--
-- read clock gets wr_2xclk in DPM otherwise, it gets rd_2xclk.
-- Use generates to avoid creating a logic gate here.
--

I: if (PMODE = 0) generate
       read_clk <= wr_2xclk;
end generate;

J: if (PMODE = 1) generate
       read_clk <= rd_2xclk;
end generate;

--
-- IF QPM and RMODE = 1, register read address inputs.
--

K: if (PMODE = 1 AND RMODE = 1) generate
   L: process(reset_n, rd_clk)
   begin
    if (reset_n = '0') then
 
      radr_a   <= (OTHERS => '0');
      radr_b   <= (OTHERS => '0');
 
    elsif (rd_clk'event and rd_clk = '1') then

      radr_a   <= a_radr after 1 ns;
      radr_b   <= b_radr after 1 ns;
 
    end if;
   end process;
end generate;

--
-- Otherwise, if DPM use the write address bits,
-- if QPM, just pass the read address to the mux
--

M: if (PMODE = 0) generate
       radr_a <= a_wadr(get_adr(RD_DEPTH) downto 0);
       radr_b <= b_wadr(get_adr(RD_DEPTH) downto 0);
end generate;

N: if (PMODE = 1 AND RMODE = 0) generate
       radr_a <= a_radr;
       radr_b <= b_radr;
end generate;

-- If QPM and RMODE = 1 or DPM and WMODE = 1
-- register the read enables

Q: if ((PMODE = 1 AND RMODE = 1) OR (PMODE = 0 AND WMODE = 1)) generate
   P: process(reset_n, read_clk)
   begin
    if (reset_n = '0') then
 
      rden_a   <= '0';
      rden_b   <= '0';
 
    elsif (read_clk'event and read_clk = '1') then

      rden_a   <= a_rden after 1 ns;
      rden_b   <= b_rden after 1 ns;
 
    end if;
   end process;
end generate;

-- otherwise pass 'em through

S: if ((PMODE = 0 AND WMODE = 0) OR (PMODE = 1 AND RMODE = 0)) generate
       rden_a <= a_rden;
       rden_b <= b_rden;
end generate;

--
-- Multiplex the read addresses and enables to the memory
-- in DMP mode, use wr_clk, otherwise use rd_clk.
--

S1: if (PMODE = 0) generate
   rden  <= rden_a when (wr_clk = '1') else rden_b;
   radr(get_adr(RD_DEPTH) downto 0) <= radr_a(get_adr(RD_DEPTH) downto 0) 
         when (wr_clk = '1') else radr_b(get_adr(RD_DEPTH) downto 0);
end generate;

S2: if (PMODE = 1) generate
   rden  <= rden_a when (rd_clk = '1') else rden_b;
   radr(get_adr(RD_DEPTH) downto 0) <= radr_a(get_adr(RD_DEPTH) downto 0) 
         when (rd_clk = '1') else radr_b(get_adr(RD_DEPTH) downto 0);
end generate;

-- Tie off unused inputs

T: if (get_adr(RD_DEPTH) < 11) generate
       radr(11 downto get_adr(RD_DEPTH)+1) <= (OTHERS => '0');
end generate;

--
-- The read data needs to be "distributed" across
-- the total of 288 possible bits based on RD_DEPTH
-- and the calculated number of output bits, and
-- allocation of 36 bits of the write bus to each
-- physical memory
--

U: for i in 0 to ((WR_WIDTH/get_bits(WR_DEPTH))-1) generate
    V: for j in 0 to (get_bits(RD_DEPTH)-1) generate       
       rd_data((get_bits(RD_DEPTH)*i)+j) <= rdata((i*36)+j);
    end generate;
end generate;

--
-- IF OREG is 1 (Registered), then create the output
-- registers, NOTE: enables are pipe-lined to account
-- for the input registers on the RAM64k36.
-- use rd_clk in QPM mode, wr_clk in DPM mode
--

W: if (PMODE = 1 AND OREG = 1) generate -- QPM

    X: process(reset_n, rd_clk)
    begin
     if (reset_n = '0') then
         a_rdata <= (OTHERS => '0');
         a_oen   <= '0';
     elsif (rd_clk'event and rd_clk = '1') then
            a_oen <= rden_a after 1 ns;

      if(a_oen = '1') then
         a_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0) after 1 ns;
      end if;
     end if;
    end process;


    Y: process(reset_n, rd_clk)
    begin
     if (reset_n = '0') then
         b_rdata <= (OTHERS => '0');
         b_oen   <= '0';
     elsif (rd_clk'event and rd_clk = '0') then
            b_oen <= rden_b after 1 ns;

      if(b_oen = '1') then
         b_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0) after 1 ns;
      end if;
     end if;
    end process;

end generate;

--
--  In dual port mode - use the write clock for output registers.
--

Z: if (PMODE = 0 AND OREG = 1) generate

    AA: process(reset_n, wr_clk)
    begin
     if (reset_n = '0') then
         a_rdata <= (OTHERS => '0');
         a_oen   <= '0';
     elsif (wr_clk'event and wr_clk = '1') then
            a_oen <= rden_a after 1 ns;

      if(a_oen = '1') then
         a_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0) after 1 ns;
      end if;
     end if;
    end process;


    AB: process(reset_n, wr_clk)
    begin
     if (reset_n = '0') then
         b_rdata <= (OTHERS => '0');
         b_oen   <= '0';
     elsif (wr_clk'event and wr_clk = '0') then
            b_oen <= rden_b after 1 ns;

      if(b_oen = '1') then
         b_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0) after 1 ns;
      end if;
     end if;
    end process;

end generate;

--
-- otherwise, assign the ouput of the 
-- memory directly to the read data ports.
--

AC: if (OREG /= 1) generate
    a_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0); -- assign to output ports
    b_rdata <= rd_data(((WR_WIDTH/get_bits(WR_DEPTH))*get_bits(RD_DEPTH)) - 1 downto 0); 
end generate;

--
-- Generate RAM BLOCK(s)
--

AD: for i in 0 to ((WR_WIDTH/get_bits(WR_DEPTH))-1) generate 
      W:RAM64K36 port map(
        WRAD0 => wadr(0), WRAD1  => wadr(1),  WRAD2  => wadr(2), 
        WRAD3 => wadr(3), WRAD4  => wadr(4),  WRAD5  => wadr(5),
        WRAD6 => wadr(6), WRAD7  => wadr(7),  WRAD8  => wadr(8),
        WRAD9 => wadr(9), WRAD10 => wadr(10), WRAD11 => wadr(11),
        WRAD12=> GND, WRAD13 => GND, WRAD14 => GND, WRAD15 => GND,

        WD0 => wdata((i*36)+ 0),  WD1 => wdata((i*36)+ 1),
        WD2 => wdata((i*36)+ 2),  WD3 => wdata((i*36)+ 3),
        WD4 => wdata((i*36)+ 4),  WD5 => wdata((i*36)+ 5),
        WD6 => wdata((i*36)+ 6),  WD7 => wdata((i*36)+ 7),
        WD8 => wdata((i*36)+ 8),  WD9 => wdata((i*36)+ 9),
        WD10=> wdata((i*36)+10),  WD11=> wdata((i*36)+11),
        WD12=> wdata((i*36)+12),  WD13=> wdata((i*36)+13),
        WD14=> wdata((i*36)+14),  WD15=> wdata((i*36)+15),
        WD16=> wdata((i*36)+16),  WD17=> wdata((i*36)+17),
        WD18=> wdata((i*36)+18),  WD19=> wdata((i*36)+19),
        WD20=> wdata((i*36)+20),  WD21=> wdata((i*36)+21),
        WD22=> wdata((i*36)+22),  WD23=> wdata((i*36)+23),
        WD24=> wdata((i*36)+24),  WD25=> wdata((i*36)+25),
        WD26=> wdata((i*36)+26),  WD27=> wdata((i*36)+27),
        WD28=> wdata((i*36)+28),  WD29=> wdata((i*36)+29),
        WD30=> wdata((i*36)+30),  WD31=> wdata((i*36)+31),
        WD32=> wdata((i*36)+32),  WD33=> wdata((i*36)+33),
        WD34=> wdata((i*36)+34),  WD35=> wdata((i*36)+35),

        WW0 => w_vec(0), WW1 => w_vec(1), WW2 => w_vec(2),
        WEN => wrenb, WCLK => wr_2xclk,

        DEPTH0 => GND, DEPTH1 => GND, DEPTH2 => GND, DEPTH3 => GND,

        RW0 => r_vec(0), RW1 => r_vec(1), RW2 => r_vec(2),
        REN => rden, RCLK => read_clk,

        RDAD0 => radr(0), RDAD1  => radr(1),  RDAD2  => radr(2), 
        RDAD3 => radr(3), RDAD4  => radr(4),  RDAD5  => radr(5),
        RDAD6 => radr(6), RDAD7  => radr(7),  RDAD8  => radr(8),
        RDAD9 => radr(9), RDAD10 => radr(10), RDAD11 => radr(11),
        RDAD12=> GND, RDAD13 => GND, RDAD14 => GND, RDAD15 => GND,

        RD0 => rdata((i*36)+ 0),  RD1 => rdata((i*36)+ 1),
        RD2 => rdata((i*36)+ 2),  RD3 => rdata((i*36)+ 3),
        RD4 => rdata((i*36)+ 4),  RD5 => rdata((i*36)+ 5),
        RD6 => rdata((i*36)+ 6),  RD7 => rdata((i*36)+ 7),
        RD8 => rdata((i*36)+ 8),  RD9 => rdata((i*36)+ 9),
        RD10=> rdata((i*36)+10),  RD11=> rdata((i*36)+11),
        RD12=> rdata((i*36)+12),  RD13=> rdata((i*36)+13),
        RD14=> rdata((i*36)+14),  RD15=> rdata((i*36)+15),
        RD16=> rdata((i*36)+16),  RD17=> rdata((i*36)+17),
        RD18=> rdata((i*36)+18),  RD19=> rdata((i*36)+19),
        RD20=> rdata((i*36)+20),  RD21=> rdata((i*36)+21),
        RD22=> rdata((i*36)+22),  RD23=> rdata((i*36)+23),
        RD24=> rdata((i*36)+24),  RD25=> rdata((i*36)+25),
        RD26=> rdata((i*36)+26),  RD27=> rdata((i*36)+27),
        RD28=> rdata((i*36)+28),  RD29=> rdata((i*36)+29),
        RD30=> rdata((i*36)+30),  RD31=> rdata((i*36)+31),
        RD32=> rdata((i*36)+32),  RD33=> rdata((i*36)+33),
        RD34=> rdata((i*36)+34),  RD35=> rdata((i*36)+35));
end generate;
 
end RTL;


              